#! /bin/csh -f
#
#   Plot "drift" of a series of PDB files, relative to each other
#
#goto skip
set nawk = /usr/bin/nawk
$nawk 'BEGIN{exit}' >& /dev/null
if($status) set nawk = `which awk`

alias nawk $nawk

if(! $?CBIN) set CBIN = "/programs/ccp4/bin"

set refcycles = 10
set rmsd      = ./scripts/rmsd

if($#argv == 1) then

endif

# find rmsd jiffy
if(! -e "$rmsd") set rmsd = `dirname $0`"/rmsd"
if(! -e "$rmsd") set rmsd = "./rmsd"
if(! -e "$rmsd") goto Deploy_rmsd

plot:
echo "<html><pre>Steps"
echo '<applet width=" 700" height=" 300" code="JLogGraph.class" codebase="'$CBIN'"><param name="table" value="'
echo ' $TABLE : Steps:'
echo ' $GRAPHS:RMSDs:A:1, 2, 3, 4: '
echo ':RMS CA:A:1, 2: '
echo ':RMS xyz:A:1, 3: '
echo ':MAX xyz:A:1, 5: '
echo ':RMS B:A:1, 4: '
echo ':MAX B:A:1, 6: $$ '
echo 'pdb  RMSD_CA    RMSD_xyz      RMSD_B  MAX_xyz        MAX_B $$ '
echo '$$ '
set count = 0
foreach X ( `ls -1rt $* ` )
    # keep track of number of PDBs processed
    # get number of current file
    set number = `echo $X | nawk '{for(i=1;i<length($1);++i){ c=substr($1,i,1); if(c ~ /[0-9]/)printf c }}'`
    # synchronize internal count with number from file
    if("$number" == "") then
	set number = $count
    else
	set count = $number
    endif
    # print it out
    echo $number | nawk '{printf "%-5d", $1}'
    # now get name of the "previous" pdb file
    if(! $?lastpdb) set lastpdb = $X
    
    cat $lastpdb $X | grep -v WAT | $rmsd xlog=on
    # now the "current" file is the "last" file for the next pdb
    set lastpdb = $X
    @ count = ( $count + 1 )
end
echo '$$'
echo '"><b>For inline graphs use a Java browser</b></applet>'

skip:
# differences from first file in list
set ref = `ls -1rt $* | head -1`

echo 'Diffs from '$ref':'
echo '<applet width=" 700" height=" 300" code="JLogGraph.class" codebase="'$CBIN'"><param name="table" value="'
echo ' $TABLE : Diffs from '$ref':'
echo ' $GRAPHS:RMSDs:A:1, 2, 3, 4: '
echo ':RMS CA:A:1, 2: '
echo ':RMS xyz:A:1, 3: '
echo ':MAX xyz:A:1, 5: '
echo ':RMS B:A:1, 4: '
echo ':MAX B:A:1, 6: $$ '
echo 'pdb  RMSD_CA    RMSD_xyz      RMSD_B  MAX_xyz        MAX_B $$ '
echo '$$ '
set count = 0
foreach X ( `ls -1rt $* ` )
    # try to get number from the filename
    set number = `echo $X | nawk '{for(i=1;i<length($1);++i){ c=substr($1,i,1); if(c ~ /[0-9]/)printf c }}'`
    # otherwise, synchronize internal count with number from file
    if("$number" == "") then
	set number = $count
    else
	set count = $number
    endif
    # print it out
    echo $number | nawk '{printf "%-5d", $1}'
    cat $ref $X | grep -v WAT | $rmsd xlog=on
    @ count = ( $count + 1 )
end
echo '$$'
echo '"><b>For inline graphs use a Java browser</b></applet>'


# diffs from LAST pdb in the list

set ref = `ls -1rt $* | tail -1`

echo 'Diffs from '$ref':'
echo '<applet width=" 700" height=" 300" code="JLogGraph.class" codebase="'$CBIN'"><param name="table" value="'
echo ' $TABLE : Diffs from '$ref':'
echo ' $GRAPHS:RMSDs:A:1, 2, 3, 4: '
echo ':RMS CA:A:1, 2: '
echo ':RMS xyz:A:1, 3: '
echo ':MAX xyz:A:1, 5: '
echo ':RMS B:A:1, 4: '
echo ':MAX B:A:1, 6: $$ '
echo 'pdb  RMSD_CA    RMSD_xyz      RMSD_B  MAX_xyz        MAX_B $$ '
echo '$$ '
set count = 0
foreach X ( `ls -1rt $* ` )
    # try to get number from the filename
    set number = `echo $X | nawk '{for(i=1;i<length($1);++i){ c=substr($1,i,1); if(c ~ /[0-9]/)printf c }}'`
    # otherwise, synchronize internal count with number from file
    if("$number" == "") then
	set number = $count
    else
	set count = $number
    endif
    # print it out
    echo $number | nawk '{printf "%-5d", $1}'
    cat $ref $X | grep -v WAT | $rmsd xlog=on
    @ count = ( $count + 1 )
end
echo '$$'
echo '"><b>For inline graphs use a Java browser</b></applet>'

if(-e ./rmsd$$) rm -f ./rmsd$$
echo "</pre></html>"
exit



















Deploy_rmsd:
if(! -e ./rmsd) then
    set rmsd = ./rmsd
else
    set rmsd = ./rmsd$$
endif

cat << EOF >! $rmsd
#! $nawk -f
#
#   Calculate RMSD of atoms with the same name in two PDB files
#
#	The PDB feild:
#           |<--- here -->|
#ATOM      1  N   ALA A 327      40.574  34.523  43.012  1.00 34.04
#
#	is used to determine if two atoms are a "pair"
#
BEGIN {
if(! atom) atom = "CA"
maxXYZ = maxdB = 0
max_atom_XYZ = max_atom_dB = 0
maxXYZ_ID = maxdB_ID = "-"
max_atom_XYZ_ID = max_atom_dB_ID = "-"
}

/^ATOM/{
    # read in values (watching for duplications)
    ID = substr(\$0,12,15)
    ++count[ID]

    if(count[ID] == 1)
    {
	# not initialized yet
	X[ID] = substr(\$0, 31, 8)+0
	Y[ID] = substr(\$0, 39, 8)+0
	Z[ID] = substr(\$0, 47, 8)+0
	B[ID] = substr(\$0, 61, 6)+0
    }
    
    if(count[ID] == 2)
    {
	++pairs
    
	# seen this before, subtract values
	dX     = X[ID] - substr(\$0, 31, 8)
	dY     = Y[ID] - substr(\$0, 39, 8)
	dZ     = Z[ID] - substr(\$0, 47, 8)
	dB[ID] = B[ID] - substr(\$0, 61, 6)
	
	# get drift (and add up squares of drifts)
	sqrD   = dX*dX + dY*dY + dZ*dZ
	dXYZ[ID] = sqrt(sqrD)

	# remember maximum shifts
	if(dXYZ[ID] > maxXYZ) {maxXYZ  = dXYZ[ID]; maxXYZ_ID = ID }
	if(dB[ID]*dB[ID] > maxdB*maxdB) {maxdB = dB[ID]; maxdB_ID = ID }

	# maintain mean-square sums	
	sumXYZ += sqrD
	sumB   += dB[ID]*dB[ID]

	# separate stats for special atom type
	if(ID ~ atom)
	{
	    ++atom_pairs

	    # maintain separate mean-square sums
	    sum_atom_XYZ += sqrD
	    sum_atom_B   += dB[ID]*dB[ID]
	    
	    # remember maximum drifts too
	    if(dXYZ[ID] > max_atom_XYZ) {max_atom_XYZ  = dXYZ[ID]; max_atom_XYZ_ID = ID }
	    if(dB[ID]*dB[ID] > max_atom_dB*max_atom_dB) {max_atom_dB = dB[ID]; max_atom_dB_ID = ID }	    
	}
	# debug output
	if(debug)
	{
	    printf("%s moved %8.4f (XYZ) %6.2f (B)\\n", ID, dXYZ[ID], dB[ID])
	}	
    }

    if(count[ID] > 2)
    {
	print "WARNING: " ID " appeared more than twice! "
    }
}


END{
    
    if((pairs+0 == 0)&&(! xlog)) 
    {
	print "no atom pairs found"
	exit
    }
    rmsXYZ = sqrt(sumXYZ/pairs)
    rmsB = sqrt(sumB/pairs)
    if(atom_pairs+0 != 0)
    {
	rms_atom_XYZ = sqrt(sum_atom_XYZ/atom_pairs)
	rms_atom_B = sqrt(sum_atom_B/atom_pairs)
    }
    

    if(! xlog) 
    {
	print pairs " atom pairs found"
	print "RMSD("atom" )= " rms_atom_XYZ " ("atom_pairs, atom " pairs)"
	print "RMSD(all)= " rmsXYZ " ("pairs" atom pairs)"
	print "RMSD(Bfac)= " rmsB

	print "MAXD(all)= " maxXYZ "\\tfor " maxXYZ_ID
	print "MAXD(Bfac)= " maxdB "\\tfor " maxdB_ID
	
	# final check for orphan atoms 
	for(id in count)
	{
	    if(count[id]<2) print "WARNING: " id " only found once"
	}
    }
    else
    {
	printf "%10.8f %10.8f %10.5f %10.8f %8.2f \\n", rms_atom_XYZ, rmsXYZ, rmsB, maxXYZ, maxdB
    }
}

EOF
chmod a+x $rmsd

goto plot
