#! /bin/csh -f
#
#	SGsearch.com	3.2.18			-James Holton   2-18-03
#
#  Space group search script for scala scripts
#
set rawfile  = raw.mtz
set script   = merge.com
set logfile  = logs/merge
set tempfile = SGsearch_temp

set SG   = P1		# read from $rawfile
set CELL		
set RES  = ""		# default to resolution in $script or $rawfile

alias nawk /usr/bin/awk
nawk 'BEGIN{exit}' >& /dev/null
if($status) alias nawk awk

# command-line arguments (1 of 2)
foreach arg ( $* )
    if(-e "$arg") then
	if("$arg" =~ *.mtz) set rawfile = "$arg"
	if("$arg" =~ *.com) set script  = "$arg"
    endif
    if("$arg" == ignore) set IGNORE
end

if((! -e "$rawfile")||(! -e "$script")) then
cat << EOF
usage: $0 [$script] [$rawfile] $SG 2.2A
where:
 $script will scale and merge
 scaling, mergeing and truncating $rawfile

 $rawfile is the unmerged mtz file

 $SG is the "root" space group i.e.
    P222 for P222, P2221, P21212, and P212121

 2.2A is the resolution for scaling and mergeing


EOF
    exit 1
endif

ls -1 ${logfile}.*.log >& /dev/null
if((! $status)&&(! $?IGNORE)) then
    echo ""
    echo "NOTICE: using stats from ${logfile}.*.log"
    echo ""
endif
if(! -e logs) mkdir logs

echo -n "evaluating alternative space groups for $rawfile with $script"

##############################################################################
# update cell and SG variables from the raw MTZ header
#
##############################################################################
echo "HEADER" | mtzdump hklin $rawfile >! $tempfile
set SG = `nawk '/Space group/{print $5}' $tempfile`
set CELL = `nawk '/Cell Dimensions/{getline;getline;print}' $tempfile`
set mtzRES = `nawk '/Resolution Range/{getline;getline;print $6}' $tempfile`

# calculate unit cell volume
echo $CELL |\
nawk 'NF==6{s=atan2(1,0)/90; A=cos(s*$4); B=cos(s*$5); G=cos(s*$6); \
 skew = 1 + 2*A*B*G - A*A - B*B - G*G ; if(skew < 0) skew = -skew;\
 printf "%.3f\n", $1*$2*$3*sqrt(skew)}' |\
cat >! ${tempfile}volume
set CELLvolume = `cat ${tempfile}volume`
rm -f ${tempfile}volume >> /dev/null

# command-line arguments (2 of 2)
foreach arg ( $* )
    if("$arg" =~ [PpCcIiFfRrHh][1-6]*) set SG = `echo $arg | nawk '{print toupper($0)}'`
    if(("$arg" =~ [0-9]*)&&("$arg" !~ [a-z,A-Z]*)) then
	# a pure number is on the command line
	if("$arg" =~ *.*) then
	    # take it as resolution
	    set RES = `echo $arg | nawk '$1+0>0.5 && $1+0<10{print $1+0}'`
	else
	    # take it as ASU size
	    set ASU = `echo $arg | nawk '{printf "%d", $1+0}'`
	endif
    endif
    if("$arg" =~ [0-9]*A) then
	# looks like a resolution
	set temp = `echo $arg | nawk '$1+0>0.5 && $1+0<10{print $1+0}'`
	if($#temp) set RES = "$temp"
    endif
end


if("$RES" != "") then
    echo " to ${RES}A"
else
    echo ""
    set RES = "$mtzRES"
endif

# extract protein-compatible space groups from CCP4 in order of increasing symmetry
#nawk '$5 ~ /^PG/ && ! /m/ && ! /bar/ && $1 < 500' $CLIBD/symop.lib >! $tempfile
cat ${CLIBD}/symop.lib |\
nawk '$5 ~ /^PG/ && ! /m/ && ! /bar/ && $1 < 500{\
	printf "%s ", $0; for(op=$2;op>0;--op){getline;printf $1 $2};print ""}' |\
nawk '{print length($NF), $0}' |\
sort -n |\
nawk '{print $2,$3,$4,$5,$6,$7}' >! $tempfile


# get crystal system of $SG
set sys = `nawk -v SG=$SG '$4==SG{print $6}' $tempfile`
set latt = `echo $SG | nawk '{print substr($1,1,1)}'`

# extract point group representatives
nawk 'substr($4,2)==substr($5,3)' $tempfile >! ${tempfile}PGs
# extract point group representatives of the same lattice type
nawk -v latt=$latt 'substr($4,1,1)==latt && substr($4,2)==substr($5,3)' $tempfile >! ${tempfile}PGs

# now get all point groups that co-index with $SG
if(("$sys" == "TRIGONAL") || ("$sys" == "HEXAGONAL")) then
    set SGs =  `nawk '$6 ~ /TRIG/{print $4} $6 ~ /HEXA/{print $4}' ${tempfile}PGs`
else
    set SGs =  `nawk -v S=$sys '$6==S{print $4}' ${tempfile}PGs`
endif
rm -f $tempfile ${tempfile}PGs

# add any extra possibilities
if("$SGs " !~ *"P1 ") set SGs = "P1 $SGs"

echo "SG      ASU(A^3)  ASU(~kD) Rmerge(%)   Brightest systematic absence"

SGcheck:
foreach SG ( $SGs )

    # get size of ASU (in A^3)
    set temp = `nawk -v SG=$SG '$4==SG && $1 < 500{print $2}' $CLIBD/symop.lib`
    set temp = `echo "$CELLvolume $temp" | nawk '$2+0!=0{print $1/$2}'`
    if("$temp" != "") set volume = "$temp"
    
    # print it out in A^3 and kD at Vm=2.5
    echo "$SG" | nawk '{printf "%-7s", $0}'
    echo "$volume" | nawk '{printf "%9d  %3d      ", $0, $0/2500}'   
    
    # remove bum log files
    if(-e ${logfile}.$SG.log) then
	grep Overall: ${logfile}.$SG.log >& /dev/null
	if(($status)||($?IGNORE)) rm -f ${logfile}.$SG.log
    endif
    
    # set up reindexing procedure
    set REINDEX = ""
    set new_SG  = "$SG"
    # check for "pseudo-spacegroups"
    set temp = `echo $SG | nawk '{print toupper($1)}'`
    if("$temp" =~ [PC]2[12][12]*) then
	set axes = ""
	if("$temp" == P2221) then
	    # P2221 with screw along longest axis
	    set axes  = "a b c"
	    set new_SG = "P2221"
	endif
	if("$temp" == P2212) then
	    # P2221 with screw along mid-length axis
	    set axes  = "b c a"
	    set new_SG = "P2221"
	endif
	if("$temp" == P2122) then
	    # P2221 with screw along shortest axis
	    set axes  = "c a b"
	    set new_SG = "P2221"
	endif
	
	if("$temp" == P21212) then
	    # P21212 with non-screw along longest axis
	    set axes  = "a b c"
	    set new_SG = "P21212"
	endif
	if("$temp" == P21221) then
	    # P21212 with non-screw along mid-length axis
	    set axes  = "b c a"
	    set new_SG = "P21212"
	endif
	if("$temp" == P22121) then
	    # P21212 with non-screw along shotest axis
	    set axes  = "c a b"
	    set new_SG = "P21212"
	endif
	    
	# cannonize these
	if("$new_SG" == "P222")    set axes = "a b c"
	if("$new_SG" == "P212121") set axes = "a b c"
	
	# decide on new axis ordering (for asymmetric orthorhombics)
	if("$axes" != "") then
	    # get current axis ordering
	    # find out what the cannonical one would be
	    # then decide how to go from current ordering to the desired one
	    echo "$CELL" | nawk '{\
		# print out current axis order \
		print $1, "h"; print $2, "k"; print $3, "l"}' |\
	    sort -n |\
	    nawk '\
		# add cannonical axis names\
		NR==1{print $0, "a"} NR==2{print $0, "b"} NR==3{print $0, "c"}' |\
	    nawk -v axes="$axes" 'BEGIN{split(axes, abc)} {\
		# write desired axis ordering in front of cannonical one \
		print abc[NR], $0}' |\
	    sort |\
	    cat >! ${tempfile}order    
	    set REINDEX = "reindex "`nawk '{printf "%s", $3} NR~/[12]/{print ","}'  ${tempfile}order`
	
	    # this should give us a mapping between any two orthorhombics
	endif
    endif

    # only run mergeing step if we havn't already done so
    if((! -e ${logfile}.$SG.log)||($?IGNORE)) then
	echo -n "" >! ${logfile}.$SG.log

	# apply the resolution cutoff
	echo "RESOLUTION $RES" |\
	mtzutils hklin1 $rawfile hklout ${tempfile}.cut.mtz >>& ${logfile}.$SG.log
	
	# now re-index the input, raw mtz file
	reindex HKLIN ${tempfile}.cut.mtz \
		HKLOUT ${tempfile}.reindexed.mtz << end_reindex >>& ${logfile}.$SG.log
SYMMETRY $new_SG
$REINDEX
END
end_reindex
	rm -f ${tempfile}.cut.mtz >& /dev/null
	
	# make sure it's sorted
	sortmtz hklout ${tempfile}.${SG}.mtz  << end_sort >>& ${logfile}.$SG.log
	H K L M/ISYM BATCH I SIGI
	${tempfile}.reindexed.mtz
end_sort
	rm -f ${tempfile}.reindexed.mtz >& /dev/null
 	
	# run the merge script
	./$script ${tempfile}.${SG}.mtz >>& ${logfile}.$SG.log
	
	# see if truncate was run
	grep "SUITE: TRUNCATE" ${logfile}.$SG.log >& /dev/null
	if($status) then
	    # truncate was not run, so lets try to run it
	    set mergedmtz = `nawk '/Filename:/{mtz=$NF} /Scala/&&/termination/{print mtz}' ${logfile}.$SG.log`
	    set ASU = `echo "$volume" | nawk '{print $0/2500}'`
	    if((-e "$mergedmtz")&&("$ASU" != "")) then
		echo "nresidue $ASU" |\
		truncate hklin $mergedmtz hklout ${tempfile}.mtz >>& ${logfile}.$SG.log
	    	if($status) then
		    # truncate failed, try it again? 
		endif
	    endif
	endif
	
	# clean up
	rm -f ${tempfile}.${SG}.mtz >& /dev/null
    endif
    
    # print out Rmerge
    set Rmerge = `nawk '/ Overall: /{print $2}' logs/merge.$SG.log`
    if("$Rmerge" == "0.000") then
	# use "Ranom" instead?
	set Rmerge = `nawk '/ Overall: /{print $5}' logs/merge.$SG.log`
    endif
    if("$Rmerge" == "") then
	echo "-"
    else
	# print out Rmerge
	echo $Rmerge | nawk '{printf "%5.1f        ", $1*100}'
    
	# Look for Systematic absence intensities
	cat  logs/merge.$SG.log |\
	nawk '/Systematic absences are OMITTED/,/Distributions/' |\
	nawk 'NF==5 && ! /[a-z]/ && $5 != 0{printf "%-4d%-4d%-4d I/sig = %8.3f\n", $1, $2, $3, $4/$5}' |\
	sort +5n >! $tempfile
	set Absence = `tail -1 $tempfile`
	if("$Absence" != "") then
	    # print out the worst one
	    tail -1 $tempfile | nawk '{printf "%s", $0}'
	endif
	rm -f $tempfile
    
	echo ""
    endif
end

# now, pick the best point/space group group
echo "" >! $tempfile
foreach SG ( $SGs )
    set Rmerge = `nawk '/ Overall: /{print $2}' logs/merge.$SG.log`
    if("$Rmerge" == "0.000") then
	# use "Ranom" instead?
	set Rmerge = `nawk '/ Overall: /{print $5}' logs/merge.$SG.log`
    endif
    if("$Rmerge" != "") then
	echo -n "$SG " >> $tempfile
	echo -n "$Rmerge " >> $tempfile
	cat  logs/merge.$SG.log |\
	nawk '/Systematic absences are OMITTED/,/Distributions/' |\
	nawk 'NF==5 && ! /[a-z]/ && $5 != 0{print $4/$5}' |\
	sort -n | tail -1 >! $tempfile.absent
	set Absence = `tail -1 $tempfile.absent`
	echo "$Absence" >> $tempfile
    endif
end
rm -f $tempfile.absent >& /dev/null


# define acceptance threshold relative to P1
set max_Rmerge = `nawk '$1=="P1" && $2+0 > 0.15 {print $2*1.5}' $tempfile`

# define acceptance threshold relative to lowest Rmerge of any SG
set max_Rmerge = `sort -n +1 $tempfile | nawk 'NF==2' | head -1 | nawk '$2>0.15{print $2+0.1}'`

# fallback to 15%
if("$max_Rmerge" == "") set max_Rmerge = 0.15
set SGs = `nawk -v max=$max_Rmerge '$2+0 < max && $3+0 < 4 {print $1}' $tempfile`

if("$SGs" == "") then
    echo "WARNING: none of these space groups merged well."
    echo "         try to tune up $script next time."
    set temp = `nawk 'BEGIN{r=9} NF>1 && r>$2+0 {r=$2+0;SG=$1} END{print "%s %d", SG, 100*r}' $tempfile`
    if($#temp != 2) then
	echo "ERROR: no space groups were acceptable."
	rm -f $tempfile
	exit
    endif
    if($temp[2] > 20) then
	echo "ERROR: no space groups were acceptable."
	rm -f $tempfile
	exit
    else
	set SGs = "$temp[1]"
	echo "Will go ahead with $SGs"
    endif
endif

# if $?PG, we have been here before
if($?PG) then
    if($#SGs == 1) then
	echo "Possible space group is: $SGs"
    else
	echo "Possible space groups are: $SGs"
    endif
#    echo "$SGs" >! SGs.txt
    rm -f $tempfile
    exit
endif
    
# extract protein-compatible space groups from CCP4 in order of increasing symmetry
#nawk '$5 ~ /^PG/ && ! /m/ && ! /bar/ && $1 < 500' $CLIBD/symop.lib >! $tempfile
cat ${CLIBD}/symop.lib |\
nawk '$5 ~ /^PG/ && ! /m/ && ! /bar/ && $1 < 500{\
        printf "%s ", $0; for(op=$2;op>0;--op){getline;printf $1 $2};print ""}' |\
nawk '{print length($NF), $0}' |\
sort -n |\
nawk '{print $2,$3,$4,$5,$6,$7}' >! $tempfile


# pursue all potential possibilites (but there should really only be one)
set temp = ""
echo "checking for screw axes in $SGs"
foreach SG ( $SGs )

    # get point group of $SG
    set PG = `nawk -v SG=$SG '$4==SG{print $5}' $tempfile`
    set latt = `echo $SG | nawk '{print substr($1,1,1)}'`
    
    # now get all space groups in $PG on $latt
    set temp =  "$temp "`nawk -v PG=$PG -v latt=$latt '$5==PG && substr($4,1,1)==latt{print $4}' $tempfile`

    # add pseudo-SGs for asymmetric orthorhombics
    if("$PG" == "PG222") then
	set pseudoSGs = ""
	foreach sg ( $temp )
	    set pseudoSGs = "$pseudoSGs $sg"
	    if("$sg" == "P2221") set  set pseudoSGs = "$pseudoSGs P2212 P2122"
	    if("$sg" == "P21212") set  set pseudoSGs = "$pseudoSGs P21221 P22121"
	end
	set temp = "$pseudoSGs"
    endif
end
rm -f $tempfile

set SGs = `echo $temp`

# check each of these

goto SGcheck


exit

