#! /bin/csh -f
#
#	./scripts/arp.com 	- intelligent arp script
#
#
#	this script adapts to most of arp's quirks
#	
#
alias nawk /usr/bin/nawk

set map_2fofc  = maps/2fofc_free.map
set map_fofc   = maps/fofc.map
set pdbin      = ./pdb/starthere.pdb	# to add water to
set pdbout     = ./pdb/arp.pdb		# new file with water added

set tempfile   = ${CCP4_SCR}arptemp

# user can override defaults
foreach arg ( $* )
    if("$arg" =~ *.mtz) set refmacmtz = "$arg"
    if("$arg" =~ *.pdb) set pdbin     = "$arg"
    if("$arg" =~ *.map) then
	if("$arg" =~ *2*) then
	    set map_2fofc = "$arg"
	else
	    set map_fofc  = "$arg"
	endif
    endif
end

# need these directories
if(! -e logs)    mkdir logs
if(! -e pdb)     mkdir pdb

if((! -e "$map_2fofc")||(! -e "$map_fofc")) then
    echo "ERROR!  no maps: $map_2fofc AND $map_fofc needed to run arp."
    exit 9
endif

ARP:
###############################################################################
#
# get ARP's variable names from map file
#
echo "" | mapdump MAPIN $map_2fofc |\
grep "Cell dimensions ..." | tail -1 |\
     nawk '{printf("%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f\n", \
              $(NF-5), $(NF-4), $(NF-3), $(NF-2), $(NF-1), $NF)}' |\
cat >! ${tempfile}cell.txt
set CELL = `cat ${tempfile}cell.txt`
#rm -f ${tempfile}dump.txt
rm -f ${tempfile}cell.txt

# change the cell in the PDB so ARP won't freak out.
pdbset XYZIN $pdbin XYZOUT ${tempfile}arpin.pdb << EOF-fixcell >& /dev/null
CELL $CELL
EOF-fixcell


# change all occupancies to be < 1 (or ARP will die)
cat ${tempfile}arpin.pdb |\
nawk '{if(substr($0,55,6)+0>1)\
          {print substr($0,1,56) "1.00" substr($0,61)}\
     else {print}}' |\
cat >! ${tempfile}arptemp.pdb
mv ${tempfile}arptemp.pdb ${tempfile}arpin.pdb


arpagain:
# Now try to run ARP
arp_warp XYZIN ${tempfile}arpin.pdb \
 MAPIN1 $map_2fofc \
 MAPIN2 $map_fofc  \
 XYZOUT $pdbout << eof-arpp | tee ${tempfile}arp.log
MODE UPDATE WATERS
CELL $CELL
SYMM P212121
RESOLUTION 74.536 2.000
! Removal based on 2mFo-DFc map
# remove about 10 waters that are below 1sigma, and merge ones closer that 2.2 A
REMOVE     ATOMS 17 CUTSIGMA 1.0 MERGE 2.2

! Addition based on mFo-DFc map
# add up to 35 atoms to chain S, with automatic sigma cuttoff
FIND       ATOMS 35 CHAIN S CUTSIGMA AUTO
# new atoms should be within 2.3 to 3.3 A of existing atoms, 
# but not closer than 2.3 A to another new atom (default).
#FDISTANCE  NEWOLD 2.3 3.3 NEWNEW 2.3

# do real-space refinement of waters
REFINE     WATERS
END
eof-arpp
if($status) goto ARPerror

# clean up after ARP
rm -f ${tempfile}arpin.pdb
rm -f ${tempfile}arpp_2fofc.map >& /dev/null
rm -f ${tempfile}arpp_fofc.map >& /dev/null

# get rid of bad water
mv $pdbout ${tempfile}.pdb
nawk '{if(! ((substr($0,55,6)=="  0.00")&&($4=="WAT"))){print} }'\
${tempfile}.pdb >! $pdbout
rm -f ${tempfile}.pdb


# sumarize for user
cat ${tempfile}arp.log |\
nawk '/New atoms found/{print "arp found  ", $2, "new waters"}'
cat ${tempfile}arp.log |\
nawk '/Number of atoms will be removed/{print "and removed", $NF, " waters"}'

echo "new pdb is in $pdbout"
rm -f ${tempfile}arp.log
exit


ARPerror:

echo "ARP died! \07 "
tail -10 ${tempfile}arp.log

# most common problem: ARPP won't swallow map 
if(! $?FIXED) then
    set FIXED
    # fix this, and go again
    set xyzlim = `nawk 'BEGIN{print "xyzlim"} /Map limits/{print $(NF-1), $NF}' ${tempfile}arp.log`
    
    if($#xyzlim == 7) then
	echo "fixed error: $xyzlim"
	echo "please edit ./scripts/maps.com and change:"
	grep xyzlim ./scripts/maps.com | head -1
	echo "to"
	echo "set xyzlim = "\"$xyzlim\"
	echo "to make this script run a bit faster"
	echo "extending the maps."
	goto extend
    endif
    
    echo "don't know why that happened! "
endif

echo "Stopping. "
echo ""
rm -f ${tempfile}arp.log

exit 8

extend:
mapmask MAPIN $map_2fofc MAPOUT ${tempfile}arpp_2fofc.map << EOF
$xyzlim
END
EOF
set map_2fofc = ${tempfile}arpp_2fofc.map
mapmask MAPIN $map_fofc MAPOUT ${tempfile}arpp_fofc.map << EOF 
$xyzlim
END
EOF
set map_fofc = ${tempfile}arpp_fofc.map

goto arpagain

