#! /bin/csh -f
#
#	Jiffy for interactively calculating cell volumes/Matthews numbers
#
#
#
alias nawk awk

# starting values
set MASS = 400000
set SG   = P1
set CELL = "100 100 100 90 90 90"
set CHAINS = 1
set tempfile = "cell_temp"

set input = "$*"
goto Setup

Return_from_Setup:

# calculate the Vm, etc
matthews_coef << end_mat >! ${tempfile}.log
CELL $CELL
SYMM $SG
MOLW $MASS
NMOL $CHAINS
END
end_mat

set Vm = `awk 'BEGIN{FS=":"} /Matthews Coefficient/{print $NF+0}' ${tempfile}.log`
set SOLC = `awk 'BEGIN{FS=":"} /the solvent %/{print $NF+0}' ${tempfile}.log`
rm -f ${tempfile}.log >& /dev/null


# report the current values
cat << EOF


CELL   $CELL
SYMM   $SG
MASS   $MASS
CHAINS $CHAINS
VM     $Vm
SOLC   ${SOLC}%
CONC   $Conc

EOF
echo "change anything? "
echo -n "-> "
set input = ( $< )

Setup:

# scan user input for changes

# detect protein mass
if("$input" =~ *aa*) then
    set MASS = `echo " $input " | nawk 'BEGIN{RS=" "} $1=="aa" && last+0>10{print last*120;exit} {last=$1}'`
endif
if("$input" =~ *atoms*) then
    set MASS = `echo " $input " | nawk 'BEGIN{RS=" "} $1=="atoms" && last+0>100{print last*7;exit} {last=$1}'`
endif
if("$input" =~ *kD*) then
    set MASS = `echo " $input " | nawk 'BEGIN{RS=" "} $1=="kD" && last+0>1{print last*1000;exit} {last=$1}'`
endif

# convert to uppercase for the rest
set input = `echo $input | nawk '{print toupper($0)}'`

if("$input" =~ *MASS*) then
    set MASS = `echo " $input " | nawk 'BEGIN{RS=" "} $1+0>100{print $1+0;exit}'`
endif
# detect chain count
if("$input" =~ *CHAIN*) then
    set CHAINS = `echo " $input " | nawk 'BEGIN{RS=" "} $1+0>0&&$1<100{print $1+0;exit}'`
endif
# detect new VM
if("$input" =~ *VM*) then
    set Vm = `echo " $input " | nawk 'BEGIN{RS=" "} $1+0>0&&$1+0<6{print $1+0;exit}'`
    set UPDATE_MASS
endif
# detect new solvent content
if(("$input" =~ *SOL*)||("$input" =~ *[0-9]%*)) then
    set SOLC = `echo " $input " | nawk 'BEGIN{RS=" "} $1+0<1{$1*=100} $1+0>1&&$1+0<100{print $1+0;exit}'`
    # change this to equivalent Vm
    set Vm = `echo $SOLC | nawk '{print 123.97/(100 - $1)}'`
    set UPDATE_MASS
endif


# detect unit cell
set cell = `echo $input | nawk '{for(i=1;i<=NF-2;++i) if(($i+0>5)&&($(i+1)+0>5)&&($(i+2)+0>5)) print $i+0, $(i+1)+0, $(i+2)+0, $(i+3)+0, $(i+4)+0, $(i+5)+0}'`
if("$input" =~ CELL*) then
    set cell = `echo "$input" | nawk '{print $2+0, $3+0, $4+0, $5+0, $6+0, $7+0}'`
endif
if($#cell == 6) then
    set CELL = `echo $cell $CELL | nawk '$1+0==0{$1=$7} $2+0==0{$2=$8} $3+0==0{$3=$9} $4+0==0{$4=$10} $5+0==0{$5=$11} $6+0==0{$6=$12} {print $1, $2, $3, $4, $5, $6}'`
endif

# detect space groups
set sg = `echo " $input " | nawk 'BEGIN{RS=" "} /[PRIFC][1-6]/{print; exit}'`
if("$sg" != "") then
    # check this word against the SG library
    if( -e $CLIBD/symop.lib) then
	set sg = `nawk -v SG=$sg '$4 == toupper(SG) {print $4}' $CLIBD/symop.lib | head -1`
    endif
    if("$sg" =~ [PpCcIiFfRr][1-6]*) then
	set SG = "$sg"
    endif
endif
# get number of assymetric units in this space group
set ASU_per_CELL = `nawk -v SG=$SG '$4 == toupper(SG) {print $2}' $CLIBD/symop.lib |& head -1`
if("$ASU_per_CELL" == "") set ASU_per_CELL = 1


# reconcile cell with this SG
set latt = `nawk -v SG="$SG" '$4 == toupper(SG) {print $6}' $CLIBD/symop.lib |& head -1`
echo "$CELL $latt" |\
nawk '{a=$1+0; b=$2+0; c=$3+0; A=$4+0; B=$5+0; G=$6+0}\
    $NF == "MONOCLINIC" { A=90; ; G=90; if($4+0 > 5) B=$4+0; if($5+0 > 5) B=$5+0}\
    $NF == "ORTHORHOMBIC" {A=90; B=90; G=90}\
    $NF == "TETRAGONAL" || $NF == "TRIGONAL" || $NF == "HEXAGONAL" {\
    b=a; A=90; B=90; G=120;\
    if((c==0) && (($2-a)^2 > .0001)) {c = $2+0}}\
    $NF == "TETRAGONAL" {G=90}\
    $NF == "CUBIC" {b=a; c=a; A=90; B=90; G=90}\
    END{if(a>5 && b>5 && c>5 && A>5 && B>5 &&G>5) print a, b, c, A, B, G}' |\
cat >! ${tempfile}
set temp = `cat ${tempfile}`
rm -f ${tempfile}
if("$#temp" == 6) then
    # actual, corrected unit cell was given
    set CELL = `echo "$temp"`
endif

# calculate the volume of the cell
echo $CELL |\
nawk 'NF==6{s=3.1415926535897899419/180; A=cos(s*$4); B=cos(s*$5); G=cos(s*$6); \
 skew = 1 + 2*A*B*G - A*A - B*B - G*G ; if(skew < 0) skew = -skew;\
 printf "%.3f\n", $1*$2*$3*sqrt(skew)}' |\
cat >! ${tempfile}volume
set CELLvolume = `cat ${tempfile}volume`
rm -f ${tempfile}volume >> /dev/null

# calculate the concentration of asymmetric units (in mol/L)
set Conc = `echo "$CELLvolume $ASU_per_CELL $CHAINS" | nawk '$2+0>0 {print $3/( ($1/$2) * (1e-10 * 10)^3 ) / 6.022e23}' | nawk '{printf "%.1f mM", $1*1000}'`



# reconcile protein mass with new VM or new SOLC
if($?UPDATE_MASS) then
    # compute an ASU size consistent with CELL and Vm provided
    set ASU = `echo "$CELLvolume $ASU_per_CELL $Vm" | nawk '$2+0>0 && $3+0>0{print ($1/$2) / $3}'`
    if("$ASU" == "") set ASU = "30000"
    
    # redefining mass, so reset chains to one
    set CHAINS = 1
    set MASS   = $ASU
endif
unset UPDATE_MASS


goto Return_from_Setup
