#! /bin/csh -f
#
#	Wedger Elves's simple Patterson script
#
#
#

set mtz1    = merged.mtz
set F1      = DANO
set SIGF1   = SIG$F1

# for isomorphous difference Patterson
set mtz2    = ""
set F2      = ""
set SIGF2   = ""

set TITLE   = ""

set outfile   = Patt.map
set ps_prefix = Patt

set tempfile = patt_temp


# optional cutoffs (determined automatically if not set)
set hiRES = ""
set loRES = ""
set DIFF  = ""
set SIG   = ""

if($#argv == 0) goto Help
goto Setup
# scan the command line for input
########################################################
Help:
cat << EOF

usage: $0 $mtz1
   or: $0 $mtz1 othermtz.mtz

where: $mtz1 is the mtz containing $F1
       othermtz.mtz is the mtz containing another, comparison F
       
EOF
rm -f ${tempfile}* >& /dev/null

exit 9
ReturnFromSetup:
########################################################
# the "Setup" prodecure should set the values of:
# $mtz1 $mtz2
# and check $F1 $SIGF1 $F2 $SIGF2
# from the command line, the defaults, or sensible deductions

# make a "working file" with exactly two F/SIGF pairs in it
cad hklin1 $mtz1 hklin2 $mtz2 hklout ${tempfile}cadded.mtz << EOF-cad
RESOLUTION OVER_ALL $loRES $hiRES

LABIN FILE 1 E1=$F1 E2=$SIGF1
LABOU FILE 1 E1=F1  E2=SIGF1
LABIN FILE 2 E1=$F2 E2=$SIGF2
LABOU FILE 2 E1=F2  E2=SIGF2

EOF-cad

# scale them together (squashing the second F for anomalous)
scaleit hklin ${tempfile}cadded.mtz \
        hklout ${tempfile}scaled.mtz << EOF-scaleit | tee ${tempfile}.log
refine isotropic
LABIN FP=F1 SIGFP=SIGF1 FPH1=F2 SIGFPH1=SIGF2
#perhaps nullify the second F
$SQUASH
EOF-scaleit

# take the suggested "DIFF", unless user already gave one
if("$DIFF" == "") set DIFF = `nawk '/acceptable differences/{print $NF}' ${tempfile}.log`
rm -f ${tempfile}scaleit.log >& /dev/null
if("$DIFF" == "") set DIFF = 1000
if("$SIG" == "")  set SIG = 0

# fix up title
set TITLE = "$TITLE excluding delta-F > $DIFF and F/sigF < $SIG"
set TITLE = `echo "$TITLE" | nawk '{print substr($0, 1, 80)}'`

# do the FFT (on the abstract variables)
fft hklin ${tempfile}scaled.mtz mapout ${tempfile}.map << EOF-fft
TITLE $TITLE
PATTERSON
LABIN F1=F1 SIG1=SIGF1 F2=F2 SIG2=SIGF2
EXCLUDE DIFF $DIFF
EXCLUDE SIG1 $SIG
EOF-fft

# normalize the map
rm -f "$outfile" >& /dev/null
echo "scale sigma" |\
mapmask mapin ${tempfile}.map  mapout $outfile 
rm -f ${tempfile}cadded.mtz >& /dev/null
rm -f ${tempfile}scaled.mtz >& /dev/null
rm -f ${tempfile}.map >& /dev/null

if(! -e "$outfile") then
    echo "WOOPS! no map produced."
    goto Help
endif
echo "$outfile is the Patterson map."

if("${ps_prefix}" == "") exit

# do a peak-pick
peakmax mapin $outfile xyzout ${ps_prefix}.pdb << EOF
THRESHOLD RMS 2
EOF


# make a PostScript version of this map?
foreach axis ( x y z )

    # permute the map axes
    echo "$axis" |\
    nawk '/x/{print "AXIS Y Z X"} /y/{print "AXIS Z X Y"} /z/{print "AXIS X Y Z"} ' |\
    mapmask mapin $outfile mapout ${tempfile}.map 

    # try to figure out good scaling
    echo "go" | mapdump mapin ${tempfile}.map >! ${tempfile}mapdump    
    cat ${tempfile}mapdump |\
    nawk '/Number of columns, rows, sections/{c=$(NF-2); r=$(NF-1)}\
      /Cell dimensions/{cell["X"]=$4;cell["Y"]=$5;cell["Z"]=$6;}\
      /Grid sampling on x, y, z/{grid["X"]=$(NF-2);grid["Y"]=$(NF-1);grid["Z"]=$NF}\
      /Fast, medium, slow axes \.\./{f=$(NF-1);m=$NF;\
         print 170/(c*cell[f]/grid[f]); print 250/(r*cell[m]/grid[m]); exit}' |\
    cat >! ${tempfile}scale
    set scale = `sort -nr ${tempfile}scale | head -1`
    rm -f ${tempfile}scale >& /dev/null
    rm -f ${tempfile}mapdump >& /dev/null
    
    npo mapin ${tempfile}.map plot ${ps_prefix}_${axis}.plot84 <<EOF
TITLE $outfile ${axis}-sections
#MAP SCALE $scale
MAP
#SECTNS 0 0
CONTRS 1 to 200 by 1
#DEVICE WIDTH 190 LENGTH 250
SIZE 100 CHAR 2 PAPER 190
PLOT
EOF

#    xplot84driver ${ps_prefix}_${axis}.plot84

    rm -f ${ps_prefix}_${axis}.ps
    pltdev -abs -por \
    -dev ps -in ${ps_prefix}_${axis}.plot84 -out ${ps_prefix}_${axis}.ps

#    xpsview ${ps_prefix}_${axis}.ps
end
rm -f ${tempfile}.map >& /dev/null

echo "$outfile is the Patterson map."
echo "${ps_prefix}_[xyz].plot84 are NPO plots of x, y, and z sections of $outfile"
echo "you should be able to view them with xplot84driver ${ps_prefix}_z.plot84 "
echo "${ps_prefix}_[xyz].ps are PostScript versions of the above plots"
exit






########################################################
Setup:
########################################################
alias nawk /usr/bin/nawk
nawk 'BEGIN{print 1; exit}' >& /dev/null
if($status) alias nawk awk

# scan command line for mtz files
set files = 0
foreach arg ( $* )
    # recognize mtz files
    if("$arg" =~ *.mtz) then
	if(! -e "$arg") then
	    if($files < 2) echo "WARNING: $arg does not exist! "
	    continue
	endif
	@ files = ( $files + 1 )
	if("$files" == 1) then
	    # cancel the defaults
	    set mtz1 = ""
	    set mtz2 = ""
	endif
	
	# check contents
	echo "go" | mtzdump hklin "$arg" |\
	 nawk 'NF>5 && $(NF-1) ~ /^[FDQ]$/ && $(NF-2) !~ /[^0-9\.]/ {\
	 print $(NF-1), $NF " "}'|\
	cat >! ${tempfile}mtzdump
	
	# make sure there's something useful in it
	egrep "^[FD] " ${tempfile}mtzdump >& /dev/null
	if($status) then
	    rm -f ${tempfile}mtzdump >& /dev/null
	    echo "WARNING: no data in $arg"
	    continue
	endif
	
	# check that Fs are actually in MTZs
	if($files == 1) then
	    egrep "[FD] $F1 " ${tempfile}mtzdump >& /dev/null
	    if($status) then
		# specified F is not in MTZ, find one
		set F1 = `nawk '/^[FD] /{print $2; exit}' ${tempfile}mtzdump`
		set SIGF1 = `nawk '/^Q /{print $2; exit}' ${tempfile}mtzdump`
		set auto_F1
	    endif
	    # this is the "first" mtz
	    set mtz1 = "$arg"
	else
	    egrep " $F2 " ${tempfile}mtzdump >& /dev/null
	    if($status) then
		# specified F is not in MTZ, find one
		set F2 = `nawk '/^[FD] /{print $2; exit}' ${tempfile}mtzdump`
		set SIGF2 = `nawk '/^Q /{print $2; exit}' ${tempfile}mtzdump`
	    endif
	    # this is the "second" mtz
	    set mtz2 = "$arg"
	endif
	rm -f ${tempfile}mtzdump >& /dev/null
	continue
    endif
end

# check for no data
if(("$mtz1" == "")||("$F1" == "")) then
    # no data
    echo "ERROR: no data"
    goto Help
endif

# check for one-file data
if(("$mtz2" == "")||("$F2" == "")) then
    # single-difference data
    if($?auto_F1) then
	# probably better to do anomalous diffs
	echo "go" | mtzdump hklin "$mtz1" |\
	 nawk 'NF>5 && $(NF-1) ~ /^[DQ]$/ && $(NF-2) !~ /[^0-9\.]/ {\
	 print $(NF-1), $NF " "}'|\
	cat >! ${tempfile}mtzdump
	# get first D, and maybe a Q
	set temp = `nawk '$1=="D"{print $2; getline; if($1=="Q") print $2}' ${tempfile}mtzdump`
	if($#temp == 2) then
	    set F1 = "$temp[1]"
	    set SIGF1 = "$temp[2]"
	endif
	if($#temp == 1) then
	    set F1 = "$temp"
	endif
    endif

    set mtz2  = "$mtz1"
    set F2    = "$F1"
    set SIGF2 = "$SIGF1"
endif

if(("$mtz1" == "$mtz2")&&("$F1" == "$F2")) then
    # single-difference data, supress second F
    set SQUASH = "SCALE FPH1 0.0000001"
    set TITLE = "$TITLE $F1"
else
    set SQUASH = ""
    set TITLE = "$TITLE ${F1}($mtz1) - ${F1}($mtz2)"
endif

# fake sigmas (if we can't find them)
if("$SIGF1" == "") set SIGF1 = "$F1"
if("$SIGF2" == "") set SIGF2 = "$F2"

goto ReturnFromSetup
